Public Class BefundEditorItem
    Implements IEditorControlItem

    Public Event ShouldBeDeleted(ByVal SelectedControl As IEditorControlItem) Implements IEditorControlItem.ShouldBeDeleted
    Public Event WasSelected(ByVal SelectedControl As IEditorControlItem) Implements IEditorControlItem.WasSelected

    Private m_Source As BefundTextEintrag
    Private m_Initializing As Boolean = False
    Private Shared VBar As New VScrollBar

    Public ReadOnly Property Source() As ChangeListItem Implements IEditorControlItem.Source
        Get
            Return m_Source
        End Get
    End Property

    Private Sub EditorItem_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Enter
        RaiseEvent WasSelected(Me)
    End Sub

    Private Sub Button1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Button1.KeyUp
        If e.KeyCode = Keys.Delete Then
            RaiseEvent ShouldBeDeleted(Me)
        End If
    End Sub


    Public Sub New()

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.

    End Sub



    Private Sub RefreshTextBoxSize()
        If Me.TextBoxDetails.Text <> "" Then
            Dim TextMeasureWidth As Single = Me.TextBoxDetails.Width - VBar.Width
            Dim g As Graphics = Me.CreateGraphics
            Dim sf As New StringFormat(StringFormat.GenericDefault)
            sf.FormatFlags = sf.FormatFlags Or StringFormatFlags.MeasureTrailingSpaces Or StringFormatFlags.FitBlackBox

            Dim Height As Single = g.MeasureString(TextBoxDetails.Text, TextBoxDetails.Font, New SizeF(TextMeasureWidth, 20000.0F), sf).Height
            Height = Math.Round(Height + 3.0F)
            Dim MaxHeight As Integer = Math.Max(38, CInt(Height))
            Me.Height = MaxHeight
            Me.TextBoxDetails.Height = MaxHeight + 1
            Me.Button1.Height = MaxHeight + 3
        Else
            Me.Height = 38
            Me.TextBoxDetails.Height = 39
            Me.Button1.Height = 41
        End If
    End Sub

    Private WithEvents m_BefundStichworte As BefundStichwortListe
    Private Sub m_BefundStichworte_Changed() Handles m_BefundStichworte.Changed
        Me.TextBoxKategorie.AutoCompleteCustomSource = m_BefundStichworte.AutoCompleteCollection
    End Sub
    Public Sub New(ByVal Source As BefundTextEintrag)
        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_Source = Source
        m_BefundStichworte = DataProvider.BefundStichworte
        Me.TextBoxKategorie.AutoCompleteCustomSource = m_BefundStichworte.AutoCompleteCollection
        Me.TextBoxKategorie.AutoCompleteMode = AutoCompleteMode.SuggestAppend
        Read_Source()

    End Sub
    Private Sub Read_Source()
        m_Initializing = True
        Me.TextBoxDetails.Text = m_Source.Text
        Me.TextBoxKategorie.Text = m_Source.Kategorie
        Me.CheckBoxWichtig.Checked = m_Source.Wichtig
        Dim MinDatum As New Date(1860, 1, 1)
        If m_Source.Datum < MinDatum Then
            Me.TextBoxDatum.Text = ""
        Else
            Me.TextBoxDatum.Text = m_Source.Datum.ToShortDateString
        End If
        If m_Source.Wichtig Then
            Me.TextBoxDetails.ForeColor = Color.Red
            Me.TextBoxKategorie.ForeColor = Color.Red
            Me.TextBoxDatum.Highlighted = True
        Else
            Me.TextBoxDetails.ForeColor = Color.Black
            Me.TextBoxKategorie.ForeColor = Color.Black
            Me.TextBoxDatum.Highlighted = False
        End If

        RefreshTextBoxSize()
        m_Initializing = False
    End Sub


    Private Sub CheckBoxWichtig_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxWichtig.CheckedChanged
        If m_Initializing Then Exit Sub
        m_Source.Wichtig = CheckBoxWichtig.Checked
        If m_Source.Wichtig Then
            Me.TextBoxDetails.ForeColor = Color.Red
            Me.TextBoxKategorie.ForeColor = Color.Red
            Me.TextBoxDatum.Highlighted = True
        Else
            Me.TextBoxDetails.ForeColor = Color.Black
            Me.TextBoxKategorie.ForeColor = Color.Black
            Me.TextBoxDatum.Highlighted = False
        End If
    End Sub

    Private Sub TextBoxKategorie_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles TextBoxKategorie.Validating
        If m_Initializing Then Exit Sub

        If m_BefundStichworte Is Nothing OrElse TextBoxKategorie.Text.Trim = "" Then
            m_Source.Kategorie = TextBoxKategorie.Text
        Else
            Dim FoundIdx As Integer = m_BefundStichworte.ShortWordsLower.IndexOf(TextBoxKategorie.Text.Trim.ToLower)
            If FoundIdx = -1 Then
                If MessageBox.Show("Neue Kategorie gefunden: " & TextBoxKategorie.Text.Trim & vbCrLf & "Soll dieses Wort dauerhaft gespeichert werden?", "Visite", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                    m_BefundStichworte.Edit(TextBoxKategorie.Text.Trim)
                End If
                m_Source.Kategorie = TextBoxKategorie.Text.Trim
            Else
                m_Source.Kategorie = m_BefundStichworte.ShortWordsNormal(FoundIdx)
                TextBoxKategorie.Text = m_Source.Kategorie
            End If

        End If

        m_Source.ValidateKategorie()
    End Sub

    Private Sub TextBoxDetails_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles TextBoxDetails.Resize
        RefreshTextBoxSize()
    End Sub

    Private Sub TextBoxDetails_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxDetails.TextChanged
        If m_Initializing Then Exit Sub
        m_Source.Text = TextBoxDetails.Text
        m_Source.ValidateText()
    End Sub

    Private Sub TextBoxDetails_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles TextBoxDetails.Validating
        If m_Initializing Then Exit Sub

        Dim ZText As String = TextBoxDetails.Text.Replace(Chr(13), " ")
        ZText = ZText.Replace(Chr(13), " ")
        ZText = ZText.Replace(Chr(10), " ")

        While ZText.IndexOf("  ") > -1
            ZText = ZText.Replace("  ", " ")
        End While
        TextBoxDetails.Text = ZText.Trim
        RefreshTextBoxSize()
        m_Source.ValidateText()
    End Sub
    Private Sub TextBoxDatum_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxDatum.TextChanged
        Dim Dat As New ParsedDate(TextBoxDatum.Text)
        m_Source.Datum = Dat.Datum
    End Sub
    Private Sub TextBoxDatum_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles TextBoxDatum.Validating
        If m_Initializing Then Exit Sub
        m_Source.ValidateDatum()
    End Sub

    Private Sub TextBoxKategorie_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxKategorie.TextChanged

    End Sub
End Class
