Public MustInherit Class TempItemList(Of T As ChangeListItem)
    Inherits List(Of T)
    Implements ITempItemList


    Public Event Changed(ByVal PropertyName As String) Implements ITempItemList.Changed
    Public Event ItemAdded(ByVal NewItem As ChangeListItem) Implements ITempItemList.ItemAdded
    Public Event SpecialItemCreated(ByVal NewItem As ExpandibleItem) Implements ITempItemList.SpecialItemCreated

    Public Sub RaiseSpecialItemCreated(ByVal NewItem As ExpandibleItem)
        RaiseEvent SpecialItemCreated(NewItem)
    End Sub


    Public Sub ValidateForNewItem()
        Dim MustAdd As Boolean = False
        If Me.Count = 0 Then
            MustAdd = True
        Else
            Dim LastItem As ChangeListItem = Me.Item(Me.Count - 1)
            If Not LastItem.IsEmpty Then
                MustAdd = True
            End If
        End If

        If MustAdd Then
            Dim NewItem As ChangeListItem = CreateNewItem()
            Me.Add(NewItem)
            RaiseEvent ItemAdded(NewItem)
        End If
    End Sub
    MustOverride Function CreateNewItem() As ChangeListItem

    Public Sub RaiseChanged(ByVal PropertyName As String)
        RaiseEvent Changed(PropertyName)
    End Sub

    Public Function CanMoveUp(ByVal Item As ChangeListItem) As Boolean Implements ITempItemList.CanMoveUp
        Dim Idx As Integer = Me.IndexOf(Item)
        If Idx = 0 Then Return False
        If Idx = Me.Count - 1 Then Return False

        Return True
    End Function
    Public Function CanMoveDown(ByVal Item As ChangeListItem) As Boolean Implements ITempItemList.CanMoveDown
        Dim Idx As Integer = Me.IndexOf(Item)
        If Idx > Me.Count - 3 Then
            Return False
        End If
        Return True
    End Function

    Public Function CanDelete(ByVal Item As ChangeListItem) As Boolean Implements ITempItemList.CanDelete
        Dim Idx As Integer = Me.IndexOf(Item)
        If Idx = Me.Count - 1 Then Return False
        Return True
    End Function

    Public Overridable Sub MoveUp(ByVal Item As ChangeListItem) Implements ITempItemList.MoveUp
        Dim Idx As Integer = Me.IndexOf(Item)
        Me.RemoveAt(Idx)
        Me.Insert(Idx - 1, Item)
        RaiseEvent Changed("")
    End Sub
    Public Overridable Sub MoveDown(ByVal Item As ChangeListItem) Implements ITempItemList.MoveDown
        Dim Idx As Integer = Me.IndexOf(Item)
        Me.RemoveAt(Idx)
        Me.Insert(Idx + 1, Item)
        RaiseEvent Changed("")
    End Sub

    Public Function GetItems() As System.Collections.Generic.List(Of ChangeListItem) Implements ITempItemList.GetItems
        Dim Result As New List(Of ChangeListItem)
        Result.AddRange(Me.ToArray)
        Return Result
    End Function

    Public Sub RemoveItem(ByVal Item As ChangeListItem) Implements ITempItemList.RemoveItem
        Me.Remove(Item)
        RaiseEvent Changed("")
    End Sub


    Public MustOverride Function CanSort() As Boolean Implements ITempItemList.CanSort
    Public MustOverride Function HeaderControl() As System.Windows.Forms.Control Implements ITempItemList.HeaderControl
    Public MustOverride Function GetColorStrings() As System.Collections.Generic.List(Of ColorString) Implements ITempItemList.GetColorStrings

    Public Overridable Sub SortList() Implements ITempItemList.SortList

    End Sub

End Class
