Public Class PatientSelector

    Private m_BMP As Bitmap
    Private WithEvents m_ItemSource As PatientSelectorSource
    Private WithEvents m_VBar As VScrollBar
    Private m_SelectedItem As PatientSelectorItem
    Public Event Selection_Changed()
    Private m_InternScroll As Boolean = False

    Private m_PaintItems As List(Of PatientSelectorItem)

    Property SelectedItem() As PatientSelectorItem
        Get
            Return m_SelectedItem
        End Get
        Set(ByVal value As PatientSelectorItem)
            If m_SelectedItem IsNot value Then
                m_SelectedItem = value
                RaiseEvent Selection_Changed()
            End If
        End Set
    End Property

    Property ItemSource() As PatientSelectorSource
        Get
            Return m_ItemSource
        End Get
        Set(ByVal value As PatientSelectorSource)
            m_ItemSource = value
            If value Is Nothing Then
                SelectedItem = Nothing
            Else
                SelectedItem = value.SelectedItem
            End If
            ReCalculate_ScrollBar()
            EnsureVisible(m_SelectedItem)
            m_BMP = Nothing
            Invalidate()
        End Set
    End Property
    Private Sub ReCalculate_ScrollBar()
        m_InternScroll = True
        m_VBar.Value = 0
        m_VBar.Visible = False

        If m_ItemSource Is Nothing Then
            m_VBar.Visible = False
        Else
            If m_ItemSource.FullHeight > Me.Height - 2 Then
                m_VBar.Visible = True
                'Der Wert einer Bildlaufleiste kann durch einen Benutzervorgang während der 
                'Laufzeit nicht seinen größtmöglichen Wert erreichen. Der größtmögliche Wert 
                'ist gleich des Maximum-Eigenschaftenwerts minus des LargeChange-
                'Eigenschaftenwerts plus 1. Der Höchstwert kann nur programmatisch erreicht werden.
                m_VBar.LargeChange = Me.Height - 2
                m_VBar.Maximum = m_ItemSource.FullHeight - 1
            Else
                m_VBar.Visible = False
            End If
        End If
        m_InternScroll = False
    End Sub

    Public Sub New()

        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.SetStyle(ControlStyles.DoubleBuffer _
                    Or ControlStyles.UserPaint _
                    Or ControlStyles.AllPaintingInWmPaint, _
                    True)
        m_VBar = New VScrollBar
        m_VBar.Location = New Point(Me.Width - 1 - m_VBar.Width, 1)
        m_VBar.Height = Me.Height - 2
        m_VBar.Anchor = AnchorStyles.Top Or AnchorStyles.Bottom Or AnchorStyles.Right
        m_VBar.Visible = True
        Me.Controls.Add(m_VBar)
    End Sub



    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim Width As Integer = Me.Width
        If m_VBar.Visible Then
            Width -= m_VBar.Width
        End If

        Dim Top As Integer = m_VBar.Value

        If m_BMP Is Nothing Then
            m_BMP = New Bitmap(Me.Width, Me.Height)
            Dim g As Graphics = Graphics.FromImage(m_BMP)
            g.Clear(Color.White)
            If m_ItemSource IsNot Nothing Then

                Dim Ft As Font = Me.Font

                m_PaintItems = m_ItemSource.GePaintItems(Top, Me.Height - 2)
                'm_ItemSource.Paint(g, m_VBar.Value, Me.Height - 2, Width, Me.Font)
                For Each Itm As PatientSelectorItem In m_PaintItems
                    Itm.Draw(g, Width, Top, Ft)
                Next

                If m_ItemSource.SelectedItem IsNot Nothing Then
                    m_ItemSource.SelectedItem.DrawSelected(g, Width, m_VBar.Value, Me.Font)
                End If
            End If
        End If

        e.Graphics.DrawImage(m_BMP, 0, 0)

        If m_HooveredItem IsNot Nothing Then
            Dim HooverCol As Color = Color.FromArgb(155, Color.Orange)
            Dim HooverBrush As New SolidBrush(HooverCol)
            Dim Hooverrect As New Rectangle(2, m_HooveredItem.Top - m_VBar.Value, Width - 4, m_HooveredItem.SelectionHeight)
            e.Graphics.FillRectangle(HooverBrush, Hooverrect)
            HooverBrush.Dispose()
        End If
        'e.Graphics.DrawString(m_MousePosition.ToString, Me.Font, Brushes.Black, 0, 0)

        ControlPaint.DrawVisualStyleBorder(e.Graphics, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    Private Sub PatientSelector_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If m_ItemSource IsNot Nothing Then
            m_ItemSource.MouseDown(e.Y + m_VBar.Value)
        End If
    End Sub

    Private m_HooveredItem As PatientSelectorItem
    Private Sub SetHooveredItem(ByVal Item As PatientSelectorItem)
        If m_HooveredItem IsNot Item Then
            m_HooveredItem = Item
            Invalidate()
        End If
    End Sub

    Private Sub PatientSelector_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.MouseLeave
        SetHooveredItem(Nothing)
    End Sub
    Private Sub PatientSelector_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove

        Dim RealY As Integer = e.Y + m_VBar.Value

        If m_PaintItems IsNot Nothing Then
            For Each Itm As PatientSelectorItem In m_PaintItems
                If Itm.Top + Itm.Height >= RealY AndAlso Itm.Top <= RealY Then
                    If Itm.CanSelect Then
                        SetHooveredItem(Itm)
                    Else
                        SetHooveredItem(Nothing)
                    End If
                    Exit For
                End If
            Next
        Else
            SetHooveredItem(Nothing)
        End If
    End Sub

    Private Sub PatientSelector_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If m_ItemSource Is Nothing Then Exit Sub
        ReCalculate_ScrollBar()
        EnsureVisible(m_ItemSource.SelectedItem)
        m_BMP = Nothing
        Invalidate()
    End Sub

    Private Sub m_ItemSource_Order_Changed() Handles m_ItemSource.Order_Changed
        RaiseEvent Selection_Changed()

        Dim OldValue As Integer = m_VBar.Value
        ReCalculate_ScrollBar()

        m_InternScroll = True
        m_VBar.Value = Math.Min(OldValue, m_VBar.Maximum - m_VBar.LargeChange)
        EnsureVisible(m_ItemSource.SelectedItem)
        m_InternScroll = False

        m_BMP = Nothing
        Invalidate()
    End Sub
    Private Sub EnsureVisible(ByVal SelItem As PatientSelectorItem)
        If SelItem Is Nothing Then Exit Sub
        If Not m_VBar.Visible Then Exit Sub

        Dim WasInternScroll As Boolean = m_InternScroll
        m_InternScroll = True

        Dim ItemTop As Integer = SelItem.GetVisibilityTop
        Dim ItemHeight As Integer = SelItem.GetVisibilityHeight
        'Der Wert einer Bildlaufleiste kann durch einen Benutzervorgang während der 
        'Laufzeit nicht seinen größtmöglichen Wert erreichen. Der größtmögliche Wert 
        'ist gleich des Maximum-Eigenschaftenwerts minus des LargeChange-
        'Eigenschaftenwerts plus 1. Der Höchstwert kann nur programmatisch erreicht werden.

        'Dim ScrollMax As Integer = m_VBar.Maximum - m_VBar.LargeChange + 1

        Dim AcBottom As Integer = m_VBar.Value + Me.Height - 2
        If ItemTop + ItemHeight > AcBottom Then
            Dim ZielWert As Integer = Math.Min(ItemTop + ItemHeight, m_VBar.Maximum - Me.Height - 2)
            m_VBar.Value = ZielWert
        End If

        ItemTop -= 3
        If ItemTop < m_VBar.Value Then
            m_VBar.Value = ItemTop
        End If

        If Not WasInternScroll Then
            m_InternScroll = False
        End If

    End Sub


    Private Sub m_ItemSource_SelectedItem_Changed() Handles m_ItemSource.SelectedItem_Changed
        SelectedItem = m_ItemSource.SelectedItem

        m_BMP = Nothing
        Invalidate()
    End Sub

    Private Sub m_VBar_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles m_VBar.Scroll
        If m_InternScroll Then Exit Sub

        m_BMP = Nothing
        Invalidate()
    End Sub
    'Public Function SetSelectedWithoutEvents(ByVal Lbl As PatientLabel) As Boolean
    '    If m_ItemSource Is Nothing Then
    '        Return False
    '    Else
    '        Dim Result As Boolean = m_ItemSource.SetSelectedWithoutEvents(Lbl)
    '        If Result Then

    '            m_InternScroll = True
    '            EnsureVisible(m_ItemSource.SelectedItem)
    '            m_InternScroll = False

    '            m_BMP = Nothing
    '            Invalidate()
    '        End If
    '        Return Result
    '    End If
    'End Function
End Class
