Public Class PatientSelectionControl

    Private m_Modus As PatientAnzeigeModi = PatientAnzeigeModi.Start
    Private WithEvents m_Provider As PatientProvider

    Private m_SelectedLabel As PatientLabel
    Private Sub SetSelected(ByVal Label As PatientLabel)
        If m_SelectedLabel IsNot Label Then
            m_SelectedLabel = Label
            m_Provider.SetActive(m_SelectedLabel)
        End If
    End Sub

    Public ReadOnly Property AllLabels() As PatientLabelGroupCollection
        Get
            Dim Result As PatientLabelGroupCollection = Me.PatientSelector1.ItemSource.AllLabels
            Select Case m_Modus
                Case PatientAnzeigeModi.ABisZ
                    Result.Title = "Patienten sortiert"
                Case PatientAnzeigeModi.Entlassene
                    Result.Title = "Entlassene Patienten"
                Case PatientAnzeigeModi.Alle
                    Result.Title = "Alle Patienten"
                Case PatientAnzeigeModi.Spezial
                    Result.Title = "Spezial-Suche"
                Case PatientAnzeigeModi.Zimmer
                    Result.Title = "Stationsbersicht"
            End Select
            Return Result
        End Get
    End Property
    Public Sub Reset()
        If m_Modus = PatientAnzeigeModi.Zimmer Then
            Me.PatientSelector1.ItemSource = New PatientStationsSelectorSource(m_SelectedLabel)
            ReadUpDown()
        End If
    End Sub
    Public Sub Init()
        m_Provider = DataProvider.PatientProvider
        SetModus(PatientAnzeigeModi.Zimmer)
    End Sub

    Private Sub SetModus(ByVal NewModus As PatientAnzeigeModi)
        If m_Modus <> NewModus Then
            m_Modus = NewModus
            Me.ToolStripButtonABisZ.Checked = False
            Me.ToolStripButtonStation.Checked = False
            Me.ToolStripButtonEntlassen.Checked = False
            Me.ToolStripButtonAlle.Checked = False
            Me.ToolStripButtonSpezial.Checked = False
            Select Case m_Modus
                Case PatientAnzeigeModi.ABisZ
                    Me.ToolStripButtonABisZ.Checked = True
                    Me.PatientSelector1.ItemSource = New PatientNormalSelectorSource(m_Modus, m_SelectedLabel)
                Case PatientAnzeigeModi.Alle
                    Me.ToolStripButtonAlle.Checked = True
                    Me.PatientSelector1.ItemSource = New PatientNormalSelectorSource(m_Modus, m_SelectedLabel)
                Case PatientAnzeigeModi.Entlassene
                    Me.ToolStripButtonEntlassen.Checked = True
                    Me.PatientSelector1.ItemSource = New PatientNormalSelectorSource(m_Modus, m_SelectedLabel)
                Case PatientAnzeigeModi.Zimmer
                    Me.ToolStripButtonStation.Checked = True
                    Me.PatientSelector1.ItemSource = New PatientStationsSelectorSource(m_SelectedLabel)
                Case PatientAnzeigeModi.Spezial
                    Me.ToolStripButtonSpezial.Checked = True
                    Me.PatientSelector1.ItemSource = New PatientSuchSelectorSource(m_SelectedLabel)
            End Select
            ReadUpDown()

        End If
    End Sub
   



    Private Sub ToolStripButtonStation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonStation.Click
        SetModus(PatientAnzeigeModi.Zimmer)
    End Sub

    Private Sub ToolStripButtonABisZ_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonABisZ.Click
        SetModus(PatientAnzeigeModi.ABisZ)
    End Sub

    Private Sub ToolStripButtonEntlassen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonEntlassen.Click
        SetModus(PatientAnzeigeModi.Entlassene)
    End Sub

    Private Sub ToolStripButtonAlle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonAlle.Click
        SetModus(PatientAnzeigeModi.Alle)
    End Sub

    Private Sub ToolStripButtonSpezial_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButtonSpezial.Click
        If DataProvider.SuchEinstellungen.Edit = True Then
            SetModus(PatientAnzeigeModi.Spezial)
        End If
    End Sub
    Public Sub Patient_Aufnehmen()
        Dim ZimmerNr As String = ""
        Dim BettNr As Integer = -1
        If Me.PatientSelector1.SelectedItem IsNot Nothing Then
            If Me.PatientSelector1.SelectedItem.GetType Is GetType(StationsPatientItem) Then
                Dim Itm As StationsPatientItem = Me.PatientSelector1.ItemSource.SelectedItem
                ZimmerNr = Itm.Bett.Zimmer.Text
                BettNr = Itm.Bett.BettNr
            ElseIf Me.PatientSelector1.SelectedItem.GetType Is GetType(StationsBettItem) Then
                Dim Itm As StationsBettItem = Me.PatientSelector1.ItemSource.SelectedItem
                ZimmerNr = Itm.Zimmer.Text
                BettNr = Itm.BettNr
            ElseIf Me.PatientSelector1.SelectedItem.GetType Is GetType(StationsZimmerItem) Then
                Dim Itm As StationsZimmerItem = Me.PatientSelector1.ItemSource.SelectedItem
                ZimmerNr = Itm.Text
            End If
        End If
        DataProvider.PatientProvider.CreateNewPatient(ZimmerNr, BettNr)
    End Sub

    Private Sub CommandPanel1_AddNew() Handles CommandPanel1.AddNew
        Patient_Aufnehmen()
    End Sub
    Private Sub Patient_Entlassen()
        DataProvider.PatientProvider.Patient_Entlassen()
    End Sub

    Private Sub m_Provider_ActivePatientAdded() Handles m_Provider.ActivePatientAdded
        Dim MinDatum As New Date(1860, 1, 1)
        Dim OldModus As PatientAnzeigeModi = m_Modus
        Dim NewModus As PatientAnzeigeModi
        m_Modus = PatientAnzeigeModi.Start
        If m_Provider.ActiveEditPatient.Entlassungsdatum < MinDatum Then
            'Patient ist nicht entlassen
            If OldModus = PatientAnzeigeModi.Entlassene Then
                NewModus = PatientAnzeigeModi.Zimmer
            Else
                NewModus = OldModus
            End If
        Else
            'Patient ist entlassen
            If OldModus = PatientAnzeigeModi.Zimmer OrElse OldModus = PatientAnzeigeModi.ABisZ Then
                NewModus = PatientAnzeigeModi.Entlassene
            Else
                NewModus = OldModus
            End If
        End If
        m_SelectedLabel = m_Provider.ActiveEditPatient.Label
        SetModus(NewModus)
        ReadUpDown()
    End Sub

    Private Sub PatientSelector1_Selection_Changed() Handles PatientSelector1.Selection_Changed

        Dim Item As PatientSelectorItem = PatientSelector1.SelectedItem
        If Item IsNot Nothing Then
            If Item.GetType Is GetType(StationsPatientItem) Then
                Dim Itm As StationsPatientItem = Item
                SetSelected(Itm.Label)
            ElseIf Item.GetType Is GetType(SortedPatientSelectorItem) Then
                Dim Itm As SortedPatientSelectorItem = Item
                SetSelected(Itm.Label)
            ElseIf Item.GetType Is GetType(PatientSuchItem) Then
                Dim Itm As PatientSuchItem = Item
                SetSelected(Itm.Label)
            Else
                SetSelected(Nothing)
            End If
        Else
            SetSelected(Nothing)
        End If
        ReadUpDown()
    End Sub
    Private Sub ReadUpDown()
        Dim Item As PatientSelectorItem = PatientSelector1.SelectedItem
        If Item Is Nothing Then
            Me.CommandPanel1.DeleteEnabled = False
            Me.CommandPanel1.MoveDownEnabled = False
            Me.CommandPanel1.MoveUpEnabled = False
            Me.CommandPanel1.EditEnabled = False
        Else
            Me.CommandPanel1.DeleteEnabled = Item.CanEntlassen
            Me.CommandPanel1.MoveDownEnabled = Item.CanMoveDown
            Me.CommandPanel1.MoveUpEnabled = Item.CanMoveUp
            Me.CommandPanel1.EditEnabled = Item.CanDelete
        End If
    End Sub

    Private Sub CommandPanel1_Delete() Handles CommandPanel1.Delete
        Patient_Entlassen()
        ReadUpDown()
    End Sub

    Private Sub CommandPanel1_MoveDown() Handles CommandPanel1.MoveDown
        Dim Item As PatientSelectorItem = PatientSelector1.SelectedItem
        If Item IsNot Nothing Then
            Item.MoveDown()
        End If
    End Sub

    Private Sub CommandPanel1_MoveUp() Handles CommandPanel1.MoveUp
        Dim Item As PatientSelectorItem = PatientSelector1.SelectedItem
        If Item IsNot Nothing Then
            Item.MoveUp()
        End If
    End Sub

End Class
